;; Generated package description from doric-themes.el  -*- mode: lisp-data; no-byte-compile: t -*-
(define-package "doric-themes" "0.2.0" "Highly legible minimalist themes with precise typography" '((emacs "29.1")) :commit "279297c36c16b02fac114aa817c59cc2f71b3d4a" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Protesilaos Stavrou" . "info@protesilaos.com") :keywords '("faces" "theme" "accessibility") :url "https://github.com/protesilaos/doric-themes")
